/**
Diese Klasse regelt prinzipiell alle Servobewegungen. 
D.h. sie schließt und öffnet das Ventil, dreht, sowie hebt und senkt den Arm.
Sie berechnet dazu den richtigen Winkel, um den Arm zum Glas zu drehen (moveArm()).
Dann öffnet sie das Ventil und füllt das Glas (fillGlass()) bis zwei Elektroden über 
die Flüssigkeit verbunden sind (glassFull()).
**/


#include "Arduino.h"
#include <Servo.h>

class Filler{
  
    private:
    //Variablen
    int distSonar;  // Distanz des Sonars zur Drehachse
    int armLength; //  Armlänge in mm
    double radiusGlass;
    
    double pi;
    
    int threshold; //  Korrekturwert damit nicht zu früh eine Spannung erkannt wird
    int electrodePin;
   
    // Arm-Servos für Drehbewegung + Heben
	int rotor;     //mittlerer Anschluss auf dem Breadboard
    int lifter;    //innerer Anschluss
    
    //  Servos
    Servo rotate;
    Servo lift;
    Servo ventil;
    
    // Servo mit Ventil-Funktion - 0 ist weitestmöglich Richtung geschlossen, 180 ist weitestmöglich geöffnet
    int vent;  //äußerer Anschluss
    int valveShut; // Gradzahl, bei welcher der Servo vollständig geschlossen ist
    int valveOpen; // Gradzahl, bei welcher der Servo vollständig geöffnet ist
    
    // Variablen, die sich die aktuelle Position der Servos merken
    int rotPos; 
    int liftPos;
    int ventPos;
    
    public: 
    /**
    	Konstruktor. Eingabewerte sind die Servos für die Drehbewegung (newRotor), 
    	die Senkbewegung (newLifter), die Ventilbewegung(newVent)
    	und ein Pin für die Elektrode zur Messung des Füllstandes.
    **/
    void setup(int newRotor, int newLifter, int newVent, int newPin){
      
      distSonar = 310; 
      armLength = 229; 
      radiusGlass = 22.5;
      
      pi= 3.15;
      
      threshold = 20; 
      
      valveShut = 65; 
      valveOpen = 180;
      
      rotPos = 175; 
      liftPos = 100;
      ventPos = valveShut;
       
      rotor = newRotor;
      lifter = newLifter;
      vent = newVent;
      
      electrodePin = newPin;
     
      // Arm-Servos anschließen und ausrichten
      rotate.attach(rotor, 800, 2100);
      lift.attach(lifter, 800, 2100);
    
      resetArm();    
    
      // Ventil-Servo anschließen und ausrichten
      ventil.attach(vent);
      servoMover(ventil, ventPos, valveShut); 
      ventPos = valveShut;
    }
    
    //wird nicht genutzt
    void loop(){
    }
  
  	//Methoden
  	  
 	/**
 	   	Methode, die den Arm in Ausgangsposition bringt.
 	**/
  	void resetArm(){ 
    	servoMover(lift, liftPos, 100);  
    	liftPos = 100;
    	servoMover(rotate, rotPos, 175);
    	rotPos = 175;
  	}

    /**
    	Diese Methode sorgt dafür, dass ein Sevo von oldPos sanft zur newPos bewegt wird.
	**/
    
    void servoMover(Servo servo, int oldPos, int newPos){
      while(oldPos != newPos){
        if(oldPos>newPos) oldPos--;
        else oldPos++;
        servo.write(oldPos);
        delay(10);
      }
      if(oldPos == newPos) servo.write(newPos);
    
    }
    
	/**
		Öffnet das Ventil, wartet bis das darunter befindliche Glas 
		voll ist, schließt das Ventil wieder.
	**/
	void fillGlass(){
		//öffne das Ventil
		servoMover(ventil, ventPos, valveOpen);
		ventPos = valveOpen;
		//warte bis Flüssigkeit Elektroden berührt
		while(!glassFull())  delay(100);
		//schließe das Ventil
		servoMover(ventil, ventPos, valveShut);
		ventPos = valveShut;
	}

	/**
		Methode, welche die Armbewegung ausführt, d.h. ausfahren und senken.
	**/
	void moveArm(double distance){

		double alpha = asin(distance/armLength); // Berechnung der Arm-Position bei gegebenem Abstand
		alpha = (alpha/pi)*180; // Umrechnung von Rad in Deg    

		delay(500L);
		//alpha ist nicht immer korrekt, also gleichen wir den Fehler mittels mapping aus
		alpha = map(alpha, 0, 90, 180, 70); 

		//Bewegung der Servos
		servoMover(rotate, rotPos, (int) alpha);
		rotPos = (int) alpha;
		delay(500L);
		servoMover(lift, liftPos, 30);
		liftPos = 30;
	}

	/**
		Methode, die anhand der Elektroden am Zapfarm erkennt, ob das darunter befindliche Glas voll ist.
		Gibt bei Kontakt der Elektroden mit der Flüssigkeit (also Glas voll) true aus, sonst false.
	**/
	boolean glassFull(){
		//Wenn Strom fließt, ist die Flüssigkeit hoch genug, um Strom zu leiten
		if(analogRead(electrodePin) <= threshold) return false; 
		else return true;
	}
	 
};
